//MCU = ATtiny13a
//F_CPU = 9600000


#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

/* Подключение файла с различными обьявлениями макросов
 * для дальнейшего использования их в программе.
 */
#include "definitions.h"

#define SPEED 100		//период опроса входного сигнала

void loadByte(uchar byte);              // передать байт на регистр


static void byte_received(uchar data);  // функция приема байта по UART
static void byteReceive();              // обработка принятого байта



// обработчик прерывания PCINT0
ISR(PCINT0_vect, ISR_BLOCK) {
    if(bit_is_clear(PINB, RX)) {
        RECEIVE_START;      // запретить прерывания PCINT0 на время приёма
        byteReceive();      // начать прием байта
    }
}



int main(void) {
    PORTB = 0b11101000;
    DDRB =  0b00000111;

    GIMSK = (1 << PCIE);
    PCMSK = (1 << RX);			//PCINT на RX

    sei();
    for(;;) {}
}


// передать байт на регистр
void loadByte(uchar byte) {
    uchar i;
    for(i = 0; i < 8; ++i) {        // Последовательная передача 8 бит на регистр
        if((byte & 0x80) == 0)
            DATA_LOW;
        else
            DATA_HIGH;
        SHIFT_CLOCK;                // Тактирующий импульс после каждого бита
        byte <<= 1;
    }
}



// функция приема байта по UART
void byteReceive(void) {
    sei();
    uchar buf = 0;      // байт в который будут последовательно записываться биты
    uchar i;

    _delay_us(SPEED/2);
    if(bit_is_clear(PINB, RX))  	// Проверка, точно ли это был стартовый бит
        buf = 0;
    else {							// Если все же помеха, то закончить прием
        RECEIVE_END;
        return;
    }

    for(i = 0; i < 8; i++) {        // Прием 8 бит по UART
        _delay_us(SPEED);
        buf >>= 1;
        if(bit_is_set(PINB, RX))
            buf |= 0b10000000;
    }

    _delay_us(SPEED);
    RECEIVE_END;
    byte_received(buf);				// Обработка полученного байта
}



// обработка принятого байта
void byte_received(uchar data) {
    loadByte(data);                 // Загрузить полученный байт в регистр
    STATE_LOAD;
}


